#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Whois::Mongo module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 34;
# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Whois::Mongo';

        use_ok('Tester');
        use_ok('Value::Convertor');
        use_ok('Value::IPv4ADDR');
        use_ok($TESTED_CLASS);
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
diag("$TESTED_CLASS version: " . $Whois::Negistry::VERSION) if Tester::D();

# Now attempt to create a class instance and check it
my $instance = $TESTED_CLASS->new('mentat', 'groups');
isa_ok($instance, $TESTED_CLASS);

# View the created instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D();

my @resultsA = $instance->find('146.102.1.1');
diag('Results A: '.Dumper(\@resultsA)) if Tester::D();
is(scalar @resultsA,                                  1,                        "Count check A01");
is($resultsA[0]->{'netname'},                         'VSE-TCZ',                "Value check A01");
is($resultsA[0]->{'first'}.":".$resultsA[0]->{'last'}, '2456158208:2456223743', "Value check A02");

my @resultsB = $instance->find('147.228.2.2');
diag('Results B: '.Dumper(\@resultsB)) if Tester::D();
is(scalar @resultsB,                                  1,                        "Count check A02");
is($resultsB[0]->{'netname'},                         'ZCU-TCZ',                "Value check A03");
is($resultsB[0]->{'first'}.":".$resultsB[0]->{'last'}, '2481192960:2481258495', "Value check A04");

my @resultsC = $instance->find('195.113.215.100');
diag('Results C: '.Dumper(\@resultsC)) if Tester::D();
is(scalar @resultsC,                                  2,                        "Count check A03");
is($resultsC[0]->{'netname'},                         'MENDELU-TCZ',            "Value check A05");
is($resultsC[0]->{'first'}.":".$resultsC[0]->{'last'}, '3279017728:3279018751', "Value check A06");

my @resultsD = $instance->rfind('abuse@vse.cz');
diag('Results D: '.Dumper(\@resultsD)) if Tester::D();
is(scalar @resultsD, 4, "Count check A04");

my @resultsE = $instance->rfind('abuse@zcu.cz');
diag('Results E: '.Dumper(\@resultsD)) if Tester::D();
is(scalar @resultsE, 3, "Count check A05");

my @resultsF = $instance->rfind('abuse@mendelu.cz');
diag('Results F: '.Dumper(\@resultsD)) if Tester::D();
is(scalar @resultsF, 9, "Count check A06");

is($instance->find_abuses('146.102.1.1'),     'abuse@vse.cz',     "Abuse check A01");
is($instance->find_abuses('147.228.2.2'),     'abuse@zcu.cz',     "Abuse check A02");
is($instance->find_abuses('195.113.215.100'), 'abuse@mendelu.cz', "Abuse check A03");
is($instance->find_abuses('195.113.134.183'), 'abuse@cesnet.cz',  "Abuse check A04");

is($instance->find_networks('abuse@vse.cz'),     '2001:718:2001::/48,2001:718:2000:1::/64,2001:718:1e02::/48,146.102.0.0-146.102.255.255', "Reverse abuse check A01");
is($instance->find_networks('abuse@zcu.cz'),     '2001:718:1801::/48,2001:718:1800:1::/64,147.228.0.0-147.228.255.255', "Reverse abuse check A02");
is($instance->find_networks('abuse@mendelu.cz'), '2001:718:803::/48,2001:718:800:3::/64,195.178.72.0-195.178.80.255,195.113.239.0-195.113.239.255,195.113.215.0-195.113.218.255,195.113.204.0-195.113.204.255,195.113.194.0-195.113.194.255,195.113.174.32-195.113.174.63,78.128.147.0-78.128.147.255', "Reverse abuse check A03");

my $instance2 = $TESTED_CLASS->new('mentat', 'groups', 1);
isa_ok($instance2, $TESTED_CLASS);

# View the created instance
diag("Instance 2 dump 01: " . Dumper($instance2)) if Tester::D();

my @resultsG = $instance2->find('146.102.1.1');
diag('Results G: '.Dumper(\@resultsG)) if Tester::D();
is(scalar @resultsG,                                  1,                        "Count check 01");
is($resultsG[0]->{'netname'},                         'VSE-TCZ',                "Value check 01");
is($resultsG[0]->{'first'}.":".$resultsG[0]->{'last'}, '2456158208:2456223743', "Value check 02");

sleep(2);

my @resultsH = $instance2->find('147.228.2.2');
diag('Results H: '.Dumper(\@resultsH)) if Tester::D();
is(scalar @resultsH,                                  1,                        "Count check 02");
is($resultsH[0]->{'netname'},                         'ZCU-TCZ',                "Value check 03");
is($resultsH[0]->{'first'}.":".$resultsH[0]->{'last'}, '2481192960:2481258495', "Value check 04");

sleep(2);

my @resultsI = $instance2->find('195.113.215.100');
diag('Results I: '.Dumper(\@resultsI)) if Tester::D();
is(scalar @resultsI,                                  2,                        "Count check 03");
is($resultsI[0]->{'netname'},                         'MENDELU-TCZ',            "Value check 05");
is($resultsI[0]->{'first'}.":".$resultsI[0]->{'last'}, '3279017728:3279018751', "Value check 06");
